package demos;

import java.security.KeyStore;
import java.security.cert.X509Certificate;

import java.net.Socket;

import java.io.FileInputStream;
import java.io.InputStream;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.TrustManagerFactory;


// #################################################
// ##                                             ##
// ## THIS PROGRAM CONNECTS TO A SERVER SOCKET    ##
// ## USING THE SSL PROTOCOL                      ##
// ##                                             ##
// ## IT ASSUMES THAT THE CLIENT KEYS ARE IN      ##
// ## ~/private/client                            ##
// ## AND THAT THE TRUSTED CERTIFICATES ARE IN    ##
// ## ~/private/clienttrust                       ##
// ## IT ALSO KNOWS THE STORE PASSWORDS AND THE   ##
// ## KEY PASSSWORD.                              ##
// ##                                             ##
// ## (c) Hernadvolgyi, 2002                      ##
// ##                                             ##
// #################################################
public class ClientDemo {

   private SSLSocketFactory sf;
   private SSLSocket s;

   public ClientDemo() throws Exception {

      // first, we need to load the keys form the keystore 
      //        (~/private/client)
      // second, we need to load the the TRUSTED certificates
      //        (~/private/clienttrust)
      SSLContext ctx ;
      KeyManagerFactory kmf;
      KeyStore cks;
      char[] cpass = "macaroni".toCharArray();

      // the SSL context HAS the keys for communication 
      //        (from ~/private/client)
      // and it knows the certificates it trusts
      //        (from ~/private/clienttrust)
      ctx = SSLContext.getInstance("TLS");
      kmf = KeyManagerFactory.getInstance("SunX509");
      cks = KeyStore.getInstance("JKS");

      // to make it easy, let's create a key manager
      cks.load(new FileInputStream("/home/liam/private/client"), cpass);
      kmf.init(cks, cpass);

      // now a trust manager, which will deal with the certificates
      TrustManagerFactory tmf;
      KeyStore sks;

      tmf = TrustManagerFactory.getInstance("SunX509");
      sks = KeyStore.getInstance("JKS");

      // load the trusted certificates
      sks.load(new FileInputStream("/home/liam/private/clienttrust"), cpass);
      tmf.init(sks);

      // initialize the SSL context
      //    now it has the communication keys 
      //    and the certificates
      ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

      // get an SSL socket factory
      sf = ctx.getSocketFactory();

   }

   // connect to the server, perform handshake, see what the 
   // server says in response
   public void connect() throws Exception {
      // get a socket connected to ecom0.site.uottawa.ca on 6249
      s = (SSLSocket)sf.createSocket("ecom0.site.uottawa.ca",6249);

      // set socket timeout!!!
      // 5 secs
      s.setSoTimeout(5000);

      s.setNeedClientAuth(true);
      // ###########################################
      // ###                                     ###
      // ### WE WANT TO PERFORM A FULL HANDSHAKE ###
      // ###                                     ###
      // ###########################################
      s.startHandshake();

      // get the socket input stream
      InputStream is = s.getInputStream();

      // dump whatever comes from the server
      int c;
      while((c = is.read()) != -1) System.out.print((char)c);
      System.out.println();

      // close this socket, it is OK if it fails
      try {
         s.close();
      } catch (Exception e) {
      }
   }


   // make a new connection to the server and get its response
   public static void main(String [] argv) throws Exception {
      (new ClientDemo()).connect();
   }
}
