package demos;

import java.security.KeyStore;
import java.security.cert.X509Certificate;

import java.net.Socket;
import java.net.ServerSocket;

import java.io.FileInputStream;
import java.io.OutputStream;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.TrustManagerFactory;


// #################################################
// ##                                             ##
// ## THIS PROGRAM LAUNCHES AN SSL SERVER SOCKET  ##
// ## IT ASSUMES THAT THE SERVER KEYS ARE IN      ##
// ## ~/private/server                            ##
// ## AND THAT THE TRUSTED CERTIFICATES ARE IN    ##
// ## ~/private/servertrust                       ##
// ## IT ALSO KNOWS THE STORE PASSWORDS AND THE   ##
// ## KEY PASSSWORD.                              ##
// ##                                             ##
// ## (c) Hernadvolgyi, 2002                      ##
// ##                                             ##
// #################################################
public class ServerDemo {

   // the socket factory
   private SSLServerSocketFactory ssf = null;

   // the actual server socket
   private SSLServerSocket ss;

   public ServerDemo() throws Exception {

      // first, we need to load the keys form the keystore 
      //        (~/private/server)
      // second, we need to load the the TRUSTED certificates
      //        (~/private/servertrust)
      SSLContext ctx;
      KeyManagerFactory kmf;
      KeyStore cks;
      char[] spass = "macaroni".toCharArray();

      // the SSL context HAS the keys for communication 
      //        (from ~/private/server)
      // and it knows the certificates it trusts
      //        (from ~/private/servertrust)
      ctx = SSLContext.getInstance("TLS");

      // to make it easy, let's create a key manager
      kmf = KeyManagerFactory.getInstance("SunX509");
      cks = KeyStore.getInstance("JKS");

      // let's load the communication keys -- or server keys
      cks.load(new FileInputStream("/home/liam/private/server"), spass);
      kmf.init(cks, spass);

      // now a trust manager, which will deal with the certificates
      TrustManagerFactory tmf;
      KeyStore sks;

      tmf = TrustManagerFactory.getInstance("SunX509");
      sks = KeyStore.getInstance("JKS");

      // load the trusted certificates
      sks.load(new FileInputStream("/home/liam/private/servertrust"), spass);
      tmf.init(sks);

      // initialize the SSL context
      //    now it has the communication keys 
      //    and the certificates
      ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

      // create an SSL socket server factory 
      ssf = ctx.getServerSocketFactory();

      // create a server socket on port 6249
      ss = (SSLServerSocket)ssf.createServerSocket(6249);

      // ########################################
      // ###                                  ###
      // ### WE WANT CLIENT AUTHORIZATION!!!! ###
      // ###                                  ###
      // ########################################
      ss.setNeedClientAuth(true);
   }

   // listen on the socket and take connections
   public void listen() throws Exception {
      for(;;) {
         SSLSocket s = (SSLSocket)ss.accept();
         // set socket timeout!!!
         // 5 secs
         s.setSoTimeout(5000);
         // spawn a new handler
         (new Connection(s)).start();
      }
   }

   // connection handler
   private class Connection extends Thread {
      private Socket s;

      private Connection(Socket s) {
         this.s = s;
      }

      public void run() {
         try {
            // get the output stream   
            OutputStream os = s.getOutputStream();
            os.write("Hello World\n".getBytes());
            os.flush();
            s.close();
         } catch(Exception e) {
            // report errors on console
            e.printStackTrace(System.out);
         } finally {
            try {
               // regardless, try to close this socket
               s.close();
            } catch (Exception e) {
            }
         }
      }
   }

   // start up a server
   public static void main(String [] argv) throws Exception {
      (new ServerDemo()).listen();
   }
}
