package demos;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.Cookie;
import javax.servlet.ServletException;

import java.io.IOException;
import java.io.PrintWriter;

import java.util.Enumeration;
import java.util.Date;

// ################################################
// ##                                            ##
// ## THIS SERVLET DEMONSTRATES COOKIES AND      ##
// ## SESSIONS.                                  ##
// ##                                            ##
// ## (c) Hernadvolgyi, 2002                     ##
// ##                                            ##
// ################################################
public class SessionDemo extends HttpServlet {


   // convenience routine
   private boolean hasValue(String param) {
      if (param == null) return false;
      return param.trim().length() > 0;
   }

   // display a value as a row of a table
   private void showValue(PrintWriter os, String name, Object val,
      String col1, String col2) throws IOException {
      os.println("<tr>");
      os.println("  <td bgcolor=\"" + col1 + "\">" + name + "</td>");
      os.println("  <td bgcolor=\"" + col2 + "\">" +
        (val == null ? "null" : val.toString()) + "</td>");
      os.println("</tr>");
   }

   // handle session request
   private void handleSession(
      HttpServletRequest request,
      HttpServletResponse response
      ) throws IOException, ServletException {

      // now we have a session
      HttpSession session = request.getSession();
 
      // if session.isNew() true, then it is a new session
      // otherwise we are still in the same session

      // get some parameters
      PrintWriter os = response.getWriter();
      String method = request.getMethod();


      response.setContentType("text/html");
      os.println("<html>");
      os.println("<head><title>Session Demo</title></head>");
      os.println("<body>");
      os.println("method: <b>" + method + "</b><br><br>");

      Object action = request.getParameter("action");

      if ("cancel".equals(action)) {
         // user canceled this session
         os.println("<font size=\"+2\">Goodbye!</font>");
         os.println("</body>");
         os.println("</html>");

         // this is how you kill a session
         session.invalidate();
         return;
      }

      os.println("<form action=\"" +
         response.encodeURL(request.getContextPath() + 
            "/servlet/demos.SessionDemo") +
         "\" method=\"" + method + "\">");
      os.println("<table border=\"4\">");
      String name;

      os.println("  <tr>");

      // Do we have a name already in the context?
      name = request.getParameter("name");
      if (hasValue(name)) {
         session.setAttribute("name",name.trim());
      } else {
         name = (String)session.getAttribute("name");
      }

      if (name == null) {
         os.println("    <td bgcolor=\"#cccccc\">Name</td>");
         os.println("    <td><input TEXT name=\"name\" size=\"12\"></td>");
      } else {
         os.println("    <td bgcolor=\"#cccccc\">Name</td>");
         os.println("    <td bgcolor=\"cyan\">" + name + "</td>");
      }
      os.println("  </tr>");
      os.println("  <tr></tr>");
      os.println("  <tr>");
      os.println("    <td bgcolor=\"#cccccc\"><b>Session Key</b></td>");
      os.println("    <td bgcolor=\"#cccccc\"><b>Session Value</b></td>");
      os.println("  </tr>");

      String newKey = request.getParameter("new_key");
      if (hasValue(newKey))  
         session.setAttribute(newKey.trim(),request.getParameter("new_value"));

      // ########## SESSION VALUES
      Enumeration E = session.getAttributeNames();
      while(E.hasMoreElements()) {
         Object key = E.nextElement();
         Object value = session.getAttribute(key.toString());
         showValue(os,(String)key,value,"#cccccc","#cccccc");
      }

      // ########## SET NEW SESSION VALUES
      os.println("  <tr></tr>");
      os.println("  <tr>");
      os.println("    <td bgcolor=\"#cccccc\"><b>New Key</b></td>");
      os.println("    <td bgcolor=\"#cccccc\"><b>New Value</b></td>");
      os.println("  </tr>");
      os.println("  <tr>");
      os.println("    <td><input TEXT name=\"new_key\" size=\"12\"></td>");
      os.println("    <td><input TEXT name=\"new_value\" size=\"12\"></td>");
      os.println("  </tr>");
      os.println("  <tr>");
      os.print("    <td bgcolor=\"gray\" >");
      os.print("<input type=\"submit\" name=\"action\"");
      os.println(" value=\"submit\"></td>");
      os.print("    <td bgcolor=\"gray\">");
      os.print("<input type=\"submit\" name=\"action\"");
      os.println(" value=\"cancel\"></td>");
      os.println("  </tr>");
      os.println("</table>");
      os.println("</form>");

      // ########## SESSION INFORMATION
      os.println("<font size=\"+2\">Session Information</font>");
      os.println("<p>");
      os.println("<table border=\"0\">");
      showValue(os,"session id",session.getId(), "cyan", "yellow");
      showValue(os,"created",new Date(session.getCreationTime()),
         "cyan", "yellow");
      showValue(os,"last accessed",new Date(session.getLastAccessedTime()),
         "cyan", "yellow");
      showValue(os,"new?",session.isNew() ? "yes" : "no",
         "cyan", "yellow");
      os.println("</table>");
      os.println("</p>");

      // ########## REQUEST INFORMATION
      os.println("<font size=\"+2\">Request Information</font>");
      os.println("<p>");
      os.println("<table border=\"0\">");
      showValue(os,"authorization", request.getAuthType(),
            "lightblue","lightgreen");
      showValue(os,"path info", request.getPathInfo(),
            "lightblue","lightgreen");
      showValue(os,"user", request.getRemoteUser(),
            "lightblue","lightgreen");
      showValue(os,"servlet path", request.getServletPath(),
            "lightblue","lightgreen");
      E = request.getParameterNames();
      while(E.hasMoreElements()) {
         String key = (String)E.nextElement();
         showValue(os,key,request.getParameter(key),
            "lightgreen","lightblue");
      }
      os.println("</table>");
      os.println("</p>");

      // ########## COOKIES
      os.println("<font size=\"+2\">Cookie Information</font>");
      os.println("<p>");
      os.println("<table border=\"0\">");
      Cookie [] cookies = request.getCookies();
      if (cookies != null) for(int i=0; i<cookies.length; ++i) {
         showValue(os, cookies[i].getName(), cookies[i].getValue(),
            "pink","navajowhite");
      }
      os.println("</table>");
      os.println("</p>");
      os.println("</body>");
      os.println("</html>");
   }

   public void doGet(
        HttpServletRequest request,   // what comes in
        HttpServletResponse response  // what goes out
      ) throws IOException, ServletException {

      handleSession(request,response);
   }

   public void doPost(
        HttpServletRequest request,   // what comes in
        HttpServletResponse response  // what goes out
      ) throws IOException, ServletException {

      handleSession(request,response);
   }
}

