package demos;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;

import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Connection;
import java.sql.DriverManager;


import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.io.FileInputStream;
import java.io.File;

import istvan.xml.XTransformer;
import javax.xml.transform.Transformer;


// ################################################
// ##                                            ##
// ## THIS SERVLET PERFORMS XML/XSLT CONVERSION  ##
// ## ON THE "SERVER-SIDE" USING THE IN-HOUSE    ##
// ## UTILITY XTransformer                       ##
// ##                                            ##
// ## (c) Hernadvolgyi, 2002                     ##
// ##                                            ##
// ################################################
public class XMLDemo extends HttpServlet {

   private static String nl = System.getProperty("line.separator");

   // standard HTML doctype
   private String doctype = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 " +
                            "Transitional//EN\">";

   // title of page
   private String title = "Simpsons";

   // header stuff
   private String top = "<html>" + nl + "<head><title>" + title +
      "</title></head>" + nl + "<body>" + nl + "<h1>" + title + "</h1>" + nl;

   // HTML trailer
   private String bottom = "</body></html>" + nl;

   // when executed on the server
   private String sDir = "c:\\samples\\work\\xml";


   public void doGet(
      HttpServletRequest request, HttpServletResponse response)
      throws IOException, ServletException {
      response.setContentType("text/html");
      doStuff("GET",response.getWriter(),sDir,request.getParameter("sheet"));
   }

   public void doPost(
      HttpServletRequest request, HttpServletResponse response)
      throws IOException, ServletException {
      response.setContentType("text/html");
      doStuff("POST",response.getWriter(),sDir,request.getParameter("sheet"));
   }


   private void doStuff(String method, PrintWriter os,
      String xslDir, String sheet)
      throws IOException {

      FileInputStream xsl = null;


      try {
         // try to figure out which style-sheet to use
         // if non given, use 1
         if (sheet != null) sheet.trim();
         if (sheet == null || sheet.equals("")) {
            sheet = "1";
         }


         // usual stuff, load the driver
         // make connection
         // execute a query
         Class.forName("com.mysql.jdbc.Driver");

         Connection C = DriverManager.getConnection(
           "jdbc:mysql://localhost/samples"
         );

         Statement s = C.createStatement();
         ResultSet R = s.executeQuery("select * from simpsons");
         C.close();

         // No results to the query
         if (R.getMetaData().getColumnCount() == 0) {
            s.close();
            throw new Exception("No result for query");
         }

         // open style sheet file
         xsl = new FileInputStream(xslDir + File.separator +
             "simpsons" + sheet + ".xsl");

         // STEP 1.
         // get an XTransformer
         XTransformer transformer = new XTransformer();


         // STEP 2.
         // get a writer
         // you write XML to this writer
         Writer writer = transformer.getWriter();

         // STEP 3.
         // write your XML to the writer
         writer.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>" + nl);
         writer.write("<!DOCTYPE Simpsons SYSTEM " +
             "\"http://localhost:8080/samples/simpsons.dtd\">" + nl);
         writer.write("<Simpsons>" + nl);


         if (R.first()) do {
            writer.write("<Simpson>" + nl);
            for(int i=1; i<= R.getMetaData().getColumnCount(); ++i) {
               String tag = R.getMetaData().getColumnName(i);
               writer.write(
                 "<" + tag + ">" + R.getObject(i) + "</" + tag + ">" + nl
               );
            }
            writer.write("</Simpson>" + nl);

         } while(R.next());
         writer.write("</Simpsons>" + nl);
         writer.flush();

         s.close(); // close statement, data already extracted

         // STEP 4.
         // this is kind of silly, unless you are debugging
         // this is a server side transform, so the XML better be
         // valid
         XTransformer.ValidateHandler handler = transformer.validate();
         if (!handler.isClean()) { // there errors
            throw new Exception("<h2>errors:</h2>" + handler.getErrors() +
               nl + "<h2>warnings:</h2>"  + handler.getWarnings() + nl);
         }

         // STEP 5.
         // transform your XML
         transformer.transform(os,xsl);

      } catch (Exception e) {

         // if anything went wrong, write HTML, not XML
         // this is what you will use for debugging!
         os.println(doctype);
         os.println(top);
         os.println("<h2>An exception occured:</h2>");
         os.println("<h3>" + e.getMessage() + "</h3>");
         os.println("<h4>CLASSPATH:" + System.getProperty("java.class.path") +
            "</h4>");
         os.println("<h3>Details:</h3>");
         os.println("<pre>");
         e.printStackTrace(os);
         os.println("</pre>");
         os.println(bottom);
         os.flush();
      } finally {
         try {
            if (xsl != null) xsl.close();
         } catch (Exception e) {
            throw new IOException(e.getMessage());
         }
      }
   }

   // command line test
   public static void main(String[] argv) throws Exception {

      if (argv.length <= 1) {
         System.out.println("directory and sheet number (1 or 2) expected");
         return;
      }

      (new XMLDemo()).doStuff("main",new PrintWriter(System.out),
         argv[0],argv[1]);
   }
}
